<?xml version="1.0" encoding="UTF-8"?>
<!-- Hey, emacs, please consider this to be -*- xml -*-

    Generates documentation as a .tex file for use with pdflatex.
  --><xsl:stylesheet xmlns:macro="http://lichteblau.com/macro" xmlns:atdoc="http://www.lichteblau.com/atdoc/" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
  <xsl:output method="text"/>

  <xsl:key name="aboutfun" match="aboutfun" use="string(.)"/>
  <xsl:key name="aboutclass" match="aboutclass" use="string(.)"/>
  <xsl:key name="aboutmacro" match="aboutmacro" use="string(.)"/>

  <xsl:key name="id" match="function-definition|class-definition|macro-definition|variable-definition" use="@id"/>

  <xsl:template match="/">
    <xsl:apply-templates select="documentation"/>
  </xsl:template>    

  <xsl:template match="documentation">
    \documentclass[a4paper]{report}
    \pagestyle{plain}
    %\usepackage{amsmath}
    %\usepackage{amssymb}
    %\usepackage{amsthm}
    %\usepackage{fancyvrb}
    \usepackage[usenames]{color}
    \usepackage[linkcolor=black,citecolor=black]{hyperref}
    \usepackage{makeidx}
    \makeindex

    \setlength{\parindent}{0cm}
    \addtolength{\parskip}{0.5em}
    %\renewcommand{\thefootnote}{\fnsymbol{footnote}}

    \begin{document}
    \input{defun.tex}

    \title{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@title)"/>}
    %\date{}
    \maketitle

    \tableofcontents
    \newpage

    <xsl:apply-templates select="package"/>

    \printindex
    \end{document}
  </xsl:template>

  <xsl:template match="arguments">
    
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Arguments</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:for-each select="arg">
	<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@arg)"/>
	--- <xsl:apply-templates/>
	<xsl:text>

</xsl:text>
      </xsl:for-each>
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
  </xsl:template>

  <xsl:template name="main-documentation-string">
    <xsl:choose>
      <xsl:when test="documentation-string">
	
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Details</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	  <xsl:apply-templates select="documentation-string"/>
	<extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      </xsl:when>
      <xsl:otherwise>
	No documentation string.  Possibly unimplemented or incomplete.
	<xsl:text>

</xsl:text>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:apply-templates select="implementation-note"/>
    <xsl:if test="see-also/constructor">
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Returned by</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
 	
    \begin{itemize}
    
	  <xsl:apply-templates select="see-also/constructor/see"/>
	
    \end{itemize}
  
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
    </xsl:if>
    <xsl:if test="see-also/slot">
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Slot Access Functions</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	
    \begin{itemize}
    
	  <xsl:apply-templates select="see-also/slot/see"/>
	
    \end{itemize}
  
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
    </xsl:if>
    <xsl:if test="//class-definition[@id=current()//superclass/@id]       //see-also       /slot">
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Inherited Slot Access Functions</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	
    \begin{itemize}
    
	  <xsl:apply-templates select="//class-definition[@id=current()//superclass/@id]        //see-also/slot/see"/>
	
    \end{itemize}
  
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
    </xsl:if>
    <xsl:if test="see-also/other|see-also/auto">
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">See also</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	
    \begin{itemize}
    
	  <xsl:for-each select="see-also/other/see|see-also/auto/see">
	    <xsl:variable name="name" select="text()"/>
	    <xsl:if test="not(preceding-sibling::see[text() = $name])">
	      <xsl:apply-templates select="."/>
	    </xsl:if>
	  </xsl:for-each>
	
    \end{itemize}
  
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
    </xsl:if>
  </xsl:template>

  <xsl:template name="class-list">
    <xsl:if test="position() != 1">
      <xsl:text>, </xsl:text>
    </xsl:if>
    <xsl:choose>
      <xsl:when test="@id">
	<extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{
	  <xsl:if test="@status = 'INTERNAL'">
	    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@package)"/>
	    <xsl:text>::</xsl:text>
	  </xsl:if>
	  <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/>
	}
      </xsl:when>
      <xsl:when test="@status = 'INTERNAL'">
	<extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\color[rgb]{0.5,0.5,0.5}</extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@package)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text><xsl:text>::</xsl:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\color[rgb]{0,0,0}</extra:text>
      </xsl:when>	  
      <xsl:otherwise>
	<extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\color[rgb]{0.5,0.5,0.5}</extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@package)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text><xsl:text>:</xsl:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\color[rgb]{0,0,0}</extra:text>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="lambda-list">
    <tt><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(../@name)"/></tt>
    <xsl:text> (</xsl:text>
    <xsl:for-each select="elt">
      <xsl:if test="position() != 1">
	<xsl:text> </xsl:text>
      </xsl:if>
      <b><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(text())"/></b>
    </xsl:for-each>
    <xsl:text>)</xsl:text>
  </xsl:template>

  <xsl:template name="slot-list">
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/>
    <xsl:text> --- </xsl:text>
    <xsl:choose>
      <xsl:when test="documentation-string/short">
        <xsl:apply-templates select="documentation-string/short"/>
	<a href="{@id}.html#details">...</a>
      </xsl:when>
      <xsl:otherwise>
        <xsl:apply-templates select="documentation-string"/>
      </xsl:otherwise>
    </xsl:choose>
    <xsl:text>

</xsl:text>
  </xsl:template>

  <xsl:template match="documentation-string">
    <div class="indent">
      <xsl:apply-templates/>
    </div>
  </xsl:template>

  <xsl:template match="short">
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="em">\emph{<xsl:apply-templates/>}</xsl:template>

  <xsl:template match="b">\textbf{<xsl:apply-templates/>}</xsl:template>

  <xsl:template match="var">\texttt{<xsl:apply-templates/>}</xsl:template>

  <!-- fixme: use verbatim? -->
  <xsl:template match="code">\texttt{<xsl:apply-templates/>}</xsl:template>

  <xsl:template match="pre">
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\begin{verbatim}</extra:text><xsl:apply-templates mode="verbatim"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\end{verbatim}</extra:text>
  </xsl:template>

  <xsl:template match="a">
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\href{</extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@a)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}{</extra:text><xsl:apply-templates/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text>
  </xsl:template>

  <xsl:template match="fun|class|slot|type|variable">
    <extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{\texttt{<xsl:apply-templates/>}}
  </xsl:template>

  <xsl:template match="itemize">
    \begin{itemize}
      <xsl:apply-templates/>
    \end{itemize}
  </xsl:template>

  <xsl:template match="item">
    \item <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="see">
    \item
    <extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{<xsl:apply-templates/>}
    <xsl:if test="@see">
      \emph{(<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@see)"/>)}
    </xsl:if>
  </xsl:template>

  <xsl:template match="implementation-note">
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text><!--
      \textbf{Implementation notes:}
--><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Implementation notes:</extra:text><xsl:apply-templates/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform"></extra:text>
    <xsl:text>

</xsl:text>
  </xsl:template>

  <xsl:template match="break">
    <xsl:text>

</xsl:text>
  </xsl:template>

  <xsl:template match="package">
    \chapter{The <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/> package}
    <xsl:apply-templates select="documentation-string"/>
    <xsl:apply-templates select="sections/section"/>
    <xsl:variable name="unreferenced" select="external-symbols/function-definition[      count(key('aboutfun',@name))=0      ]"/>
    <xsl:if test="$unreferenced">
      \section{Other functions}
      <xsl:apply-templates select="$unreferenced">
	<xsl:sort select="@id" data-type="text" order="ascending"/>
      </xsl:apply-templates>
    </xsl:if>

    <xsl:variable name="unreferenced2" select="external-symbols/macro-definition[      count(key('aboutmacro',@name))=0      ]"/>
    <xsl:if test="$unreferenced2">
      \section{Other macros}
      <xsl:apply-templates select="$unreferenced2">
	<xsl:sort select="@id" data-type="text" order="ascending"/>
      </xsl:apply-templates>
    </xsl:if>

    <xsl:variable name="unreferenced3" select="external-symbols/class-definition[      count(key('aboutclass',@name))=0      ]"/>
    <xsl:if test="$unreferenced3">
      \section{Other classes}
      <xsl:apply-templates select="$unreferenced3">
	<xsl:sort select="@id" data-type="text" order="ascending"/>
      </xsl:apply-templates>
    </xsl:if>

    <xsl:variable name="unreferenced4" select="external-symbols/variable-definition"/>
    <xsl:if test="$unreferenced4">
      \section{Other variables}
      <xsl:apply-templates select="$unreferenced4">
	<xsl:sort select="@id" data-type="text" order="ascending"/>
      </xsl:apply-templates>
    </xsl:if>
  </xsl:template>

  <xsl:template match="section">
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\</extra:text><xsl:for-each select="ancestor::section">sub</xsl:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">section{</extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@section)"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">} \label{</extra:text><extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(generate-id())"/><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text>
    <xsl:if test="aboutfun|aboutmacro|aboutclass">
      In this section:
      \begin{itemize}
      <xsl:for-each select="aboutfun|aboutmacro|aboutclass">
	\item
	<xsl:choose>
	  <xsl:when test="local-name() = 'aboutfun'">
	    <xsl:for-each select="//function-definition[@name=current()]">
	      <extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/>}
	    </xsl:for-each>
	  </xsl:when>

	  <xsl:when test="local-name() = 'aboutmacro'">
	    <xsl:for-each select="//macro-definition[@name=current()]">
	      <extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/>}
	    </xsl:for-each>
	  </xsl:when>

	  <xsl:when test="local-name() = 'aboutclass'">
	    <xsl:for-each select="//class-definition[@name=current()]">
	      <extra:if xmlns:extra="http://www.w3.org/1999/XSL/Transform" test="count(key('id', @id)) = 0"><extra:message>
	Unresolved reference to <extra:value-of select="@id"/></extra:message></extra:if>\hyperref[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>]{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/>}
	    </xsl:for-each>
	  </xsl:when>
	</xsl:choose>
      </xsl:for-each>
      \end{itemize}
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="return">
    
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Return Values</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:apply-templates/>
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
  </xsl:template>

  <xsl:template match="aboutfun">
    <xsl:apply-templates select="//function-definition[@name=current()]"/>
  </xsl:template>

  <xsl:template match="aboutmacro">
    <xsl:apply-templates select="//macro-definition[@name=current()]"/>
  </xsl:template>

  <xsl:template match="aboutclass">
    <xsl:apply-templates select="//class-definition[@name=current()]      | //type-definition[@name=current()]"/>
  </xsl:template>

  <xsl:template match="function-definition">
    <extra:variable xmlns:extra="http://www.w3.org/1999/XSL/Transform" name="d0d0e1e67e1-label">Function</extra:variable>

    \rule{\linewidth}{0.1mm}
    
    \label{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>}
    \begin{defun}[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string($d0d0e1e67e1-label)"/>]
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:for-each xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="lambda-list/elt"><extra:text xml:space="preserve"> </extra:text><extra:value-of select="."/></extra:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:apply-templates select="arguments"/>
      <xsl:apply-templates select="return"/>
      <xsl:call-template name="main-documentation-string"/>
    
    \end{defun}
  
  </xsl:template>

  <xsl:template match="macro-definition">
    <extra:variable xmlns:extra="http://www.w3.org/1999/XSL/Transform" name="d0d0e1e69e1-label">Macro</extra:variable>

    \rule{\linewidth}{0.1mm}
    
    \label{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>}
    \begin{defun}[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string($d0d0e1e69e1-label)"/>]
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:for-each xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="lambda-list/elt"><extra:text xml:space="preserve"> </extra:text><extra:value-of select="atdoc:escape-latex-string(.)"/></extra:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:apply-templates select="arguments"/>
      <xsl:apply-templates select="return"/>
      <xsl:call-template name="main-documentation-string"/>
    
    \end{defun}
  
  </xsl:template>

  <xsl:template match="class-definition">
    <extra:variable xmlns:extra="http://www.w3.org/1999/XSL/Transform" name="d0d0e1e71e1-label">Class</extra:variable>

    \rule{\linewidth}{0.1mm}
    
    \label{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>}
    \begin{defun}[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string($d0d0e1e71e1-label)"/>]
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:for-each xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="lambda-list/elt"><extra:text xml:space="preserve"> </extra:text><extra:value-of select="atdoc:escape-latex-string(.)"/></extra:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Superclasses</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	<xsl:for-each select="cpl/superclass">
	  <xsl:call-template name="class-list"/>
	</xsl:for-each>
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Documented Subclasses</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	<xsl:choose>
	  <xsl:when test="subclasses/subclass">
	    <xsl:for-each select="subclasses/subclass">
	      <xsl:sort select="@id" data-type="text" order="ascending"/>
	      <xsl:call-template name="class-list"/>
	    </xsl:for-each>
	  </xsl:when>
	  <xsl:otherwise>
	    None
	  </xsl:otherwise>
	</xsl:choose>
      <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:if test="direct-slots">
	
    \bigskip
    <extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">\textsc{</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">Direct Slots</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform">}</extra:text><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
	  <xsl:choose>
	    <xsl:when test="direct-slots/slot">
	      <xsl:for-each select="direct-slots/slot">
		<xsl:sort select="@id" data-type="text" order="ascending"/>
		<xsl:call-template name="slot-list"/>
	      </xsl:for-each>
	    </xsl:when>
	    <xsl:otherwise>
	      None
	    </xsl:otherwise>
	  </xsl:choose>
	<extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      </xsl:if>
      <xsl:call-template name="main-documentation-string"/>
    
    \end{defun}
  
  </xsl:template>

  <xsl:template match="type-definition">
    <extra:variable xmlns:extra="http://www.w3.org/1999/XSL/Transform" name="d0d0e1e73e1-label">Type</extra:variable>

    \rule{\linewidth}{0.1mm}
    
    \label{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>}
    \begin{defun}[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string($d0d0e1e73e1-label)"/>]
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:for-each xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="lambda-list/elt"><extra:text xml:space="preserve"> </extra:text><extra:value-of select="atdoc:escape-latex-string(.)"/></extra:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:call-template name="main-documentation-string"/>
    
    \end{defun}
  
  </xsl:template>

  <xsl:template match="variable-definition">
    <extra:variable xmlns:extra="http://www.w3.org/1999/XSL/Transform" name="d0d0e1e75e1-label">Variable</extra:variable>

    \rule{\linewidth}{0.1mm}
    
    \label{<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="@id"/>}
    \begin{defun}[<extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string($d0d0e1e75e1-label)"/>]
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(@name)"/><extra:for-each xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="lambda-list/elt"><extra:text xml:space="preserve"> </extra:text><extra:value-of select="atdoc:escape-latex-string(.)"/></extra:for-each><extra:text xmlns:extra="http://www.w3.org/1999/XSL/Transform" xml:space="preserve">

</extra:text>
      <xsl:call-template name="main-documentation-string"/>
    
    \end{defun}
  
  </xsl:template>

  <xsl:template name="undocumented">
    <xsl:if test="not(documentation-string)">
      <xsl:text>\color[rgb]{1,0,0}~(undocumented)</xsl:text>
    </xsl:if>
  </xsl:template>

  <xsl:template match="text()">
    <extra:value-of xmlns:extra="http://www.w3.org/1999/XSL/Transform" select="atdoc:escape-latex-string(.)"/>
  </xsl:template>

  <xsl:template match="text()" mode="verbatim">
    <!-- fixme: is there anything we need to escape here? -->
    <xsl:value-of select="string(.)"/>
  </xsl:template>
</xsl:stylesheet>